package org.processmining.uncertaintymarker.algorithms;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XLogImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.uncertaintymarker.help.Utilities;
import org.processmining.uncertaintymarker.models.SuccessorMatrix;


public class HmmPruning {

	ArrayList<Map.Entry<String, ArrayList<Map.Entry<String,Double>> >> successorTable;
	
	XLog inputLog;
	public XLog outputLog;
	
	ArrayList< Map.Entry<String, Double> > timeUpdate = new ArrayList<>();

	ArrayList< Map.Entry<String, Double> > observation = new ArrayList<>();
	
	ArrayList< Map.Entry<String, Double> > resultProbabilities = new ArrayList<>();
	int numberOfClasses;
	
	public HmmPruning(SuccessorMatrix sm, XLog log){
		
		this.successorTable=sm.successorTable;
		inputLog = log;
		
		numberOfClasses= successorTable.size();
		
		//fill timeUpdate for the first time
		for(Map.Entry<String, ArrayList<Map.Entry<String,Double>>> e : successorTable){
			timeUpdate.add(new AbstractMap.SimpleEntry<String, Double>(e.getKey(),1.0/numberOfClasses));
		}
		
		outputLog = new XLogImpl(new XAttributeMapImpl());
		
		//for each trace
		for(XTrace trace: log){
			int rowCount=0;
			
			XTraceImpl t = new XTraceImpl(new XAttributeMapImpl());

			XAttributeLiteralImpl xatt = (XAttributeLiteralImpl) trace.getAttributes().get("concept:name");
			t.getAttributes().put("concept:name", xatt);
			
			outputLog.add(t);

			//for each row
			for(XEvent row: trace){
				rowCount++;
				
				//for each column in row
				XAttributeMapImpl mp  = (XAttributeMapImpl) row.getAttributes();
				for (Map.Entry<String, XAttribute> entry : mp.entrySet()){

					//check attribute type
					XAttributeContinuousImpl columnInRow;
					if(entry.getValue() instanceof XAttributeContinuousImpl){
						columnInRow = (XAttributeContinuousImpl) entry.getValue();
					}
					else{
						continue;
					}
					
					// first part of computation 
					for(Map.Entry<String, Double> entry2 : timeUpdate){
						if(columnInRow.getKey().equals(entry2.getKey())){
							observation.add(new AbstractMap.SimpleEntry(entry2.getKey(),entry2.getValue()*columnInRow.getValue()));
							break;
						}
					}
				}
//				System.out.println();
				
				//second part of computation (normalize)
				double sum=0;
				for(Map.Entry<String, Double> e : observation){
					sum+=e.getValue();
				}
				for(Map.Entry<String, Double> e : observation){
					e.setValue(Utilities.round(e.getValue()/sum,3));
				}
				
//				System.out.println();
				
				//third part of computation (including successor relation)
				double result=0.0;
				for(Map.Entry<String, Double> tU : timeUpdate){
					for(Map.Entry<String, Double> o : observation){
						for(Map.Entry<String, ArrayList<Map.Entry<String,Double>> >  sT : successorTable){
							if(sT.getKey().equals(o.getKey())){
								for(Map.Entry<String, Double> pair : sT.getValue()){
									if(pair.getKey().equals(tU.getKey())){
										double tmp = pair.getValue()*o.getValue();
										result+= tmp;
										break;
									}
								}
							}
						}
					}
					tU.setValue(result);
					result=0.0;
				}
//				System.out.println();
				
				XAttributeMap map = new XAttributeMapImpl();
				
				for(Map.Entry<String, Double> tU : timeUpdate){
					map.put(tU.getKey(), new XAttributeContinuousImpl(tU.getKey(),tU.getValue()));
				}
				XEvent e = new XEventImpl(map);
				t.add(e);
			}
		}
	}
}
