package org.processmining.uncertaintymarker.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.text.NumberFormat;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.uncertaintymarker.parameters.Parameters;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;

/**This class is a predefined component of the ProM Framework. It basically provides a JPanel in which JComponents can be placed,
 * in order to let the user make some choices upon the different parameter settings that are needed by the plugin. 
 * 
 * @author Oliver
 *
 */


public class UserInput extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -60087716353524468L;

	public static NiceSlider asdSlider = null;
	public static JComboBox<String> pruningDropdown = null;
	
    JLabel chunkMinimumLabel;
    JFormattedTextField chunkMinimum;
    
	final NiceSlider probabilitySlider;
	
	/**
	 * The JPanel that allows the user to set (a subset of) the parameters.
	 */
	public UserInput(UIPluginContext context, XLog input1, final Parameters parameters) {
							//labels1,textfields1,space1,labels2,textfields2,space2,labels,textfields,remaining space
		double size[][] = { {130,30,50,130,30,50,130,30,160}, {40,5,30,5,40,10,30,30,30,30,10,40,5,30,30,40,30,30} };
		setLayout(new TableLayout(size));
		
		int row =0;
		
		//set defaults
		
		parameters.setIndelScore(-1);
		
		parameters.setSubScore1(1);
		parameters.setSubScore2(-1);
		parameters.setSubScore3(-1);
		
		parameters.setEvalScore1(10);
		parameters.setEvalScore2(-10);
		parameters.setEvalScore3(-1);
		parameters.setEvalScore4(-1);
		
		parameters.setAlgorithm("alignedStepsDiscovery");
		parameters.setAsdInclusionThreshold(0.25);
		
		//STEP 1 - PREPROCESSING
//		final ProMTextArea header1 = new ProMTextArea(false);
//		final JTextArea header1 = new JTextArea();
		final JLabel header1 = new JLabel("Step 1: Preprocessing / Time Window Consolidation");
//		header1.setText("Step 1: Preprocessing / Time Window Consolidation");
		add(header1,"0,"+row+","+ (size[0].length-1) +","+row); 
		row++;
		Font font = header1.getFont();
		Font boldFont = new Font(font.getFontName(), Font.BOLD, font.getSize());
		header1.setFont(boldFont);
		header1.setPreferredSize(new Dimension(20, 20));
		
		useAsHeader(header1);
		
		header1.setToolTipText("<html>First the fixed-duration time window rows coming directly from the event classification need to be aggregated to events (eg. E,E,E,E,E -> E).<br> The probabilities in the rows are aggregated and averaged. <br> " 
			+ "The result of this Time Window Consolidation is the Concise Log, containing real events how it is typical for event logs.</html>");

		row++;
		
		final JCheckBox chunkingCheckBox = SlickerFactory.instance().createCheckBox("Chunking", true);
		parameters.setChunkingActive(true);
		chunkingCheckBox.setSelected(parameters.isChunkingActive());
		chunkingCheckBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				parameters.setChunkingActive(chunkingCheckBox.isSelected());
				
				if(chunkingCheckBox.isSelected())
					pruningDropdown.setVisible(true);
				else
					pruningDropdown.setVisible(false);
				
				repaint();
			}
		});
		chunkingCheckBox.setOpaque(false);
//		checkBox.setPreferredSize(new Dimension(100, 30));
		add(chunkingCheckBox, "0, "+row+", 1, "+row);
		
		chunkingCheckBox.setToolTipText("Deactivate Preprocessing here, if log contains probability distributions over whole events instead of time windows.");
		
		String[] pruningOptions = {"Low-Frequency (Post-Pruning)","HMM (Pre-Pruning)"};
		pruningDropdown = new JComboBox<String>(pruningOptions);
		parameters.setPruningAlgorithm("Low-Frequency (Post-Pruning)");
		pruningDropdown.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e){
				String selection = (String)pruningDropdown.getSelectedItem();
				parameters.setPruningAlgorithm(selection);
				if(selection.equals("Low-Frequency (Post-Pruning)")){
					chunkMinimum.setVisible(true);
					chunkMinimumLabel.setVisible(true);
				}
				else{
					chunkMinimum.setVisible(false);
					chunkMinimumLabel.setVisible(false);
				}
			}

		});
		add(pruningDropdown,"2,"+(row)+", 4,"+(row));
		
		pruningDropdown.setToolTipText("<html>Method selection for excluding very short time window sequences. <br> If events are represented by many time windows, choose HMM.</html>");
		
		//number textfields
		NumberFormat format = NumberFormat.getInstance();
		NumberFormatter formatter = new NumberFormatter(format);
		formatter.setValueClass(Integer.class);
		formatter.setAllowsInvalid(true);
		
	    chunkMinimumLabel = new JLabel("Chunk Minimum:");
		add(chunkMinimumLabel,"6,"+(row));
		//indelLabel.setSize(new Dimension(50, 50));
	    chunkMinimum = new JFormattedTextField(formatter);
	    add(chunkMinimum,"7,"+row);
	    chunkMinimum.setValue(2);
	    parameters.setChunkMinimum(2);
	    
	    chunkMinimum.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(chunkMinimum.getText());
				chunkMinimum.setValue(tmp);
				parameters.setChunkMinimum((Integer)chunkMinimum.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
		
		row++;
		row++;	
	 
		//STEP 2 - TRACE ALIGNMENT
		final JLabel header2 = new JLabel("Step 2: Trace Alignment");
		header2.setText("Step 2: Trace Alignment");
		add(header2,"0,"+row +","+ (size[0].length-1) +","+row); 
		row+=2;
		header2.setFont(boldFont);
		
		useAsHeader(header2);
		
		header2.setToolTipText("<html> Align the different traces in the concise log, in order to produce better discovery results in the next step. <br> "
			+ "As the Trace Alignment plugin of J.C. Bose was not working at development time, I reimplemented it using their paper.</html>");

	    
	    final JLabel indelLabel = new JLabel("indelScore:");
		add(indelLabel,"0,"+row);
		//indelLabel.setSize(new Dimension(50, 50));
	    final JFormattedTextField indelScore = new JFormattedTextField(formatter);
	    add(indelScore,"1,"+row);
	    indelScore.setValue(-1);
	    
	    
	 indelScore.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(indelScore.getText());
				indelScore.setValue(tmp);
				parameters.setIndelScore((Integer)indelScore.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
		//indelScore.setSize(new Dimension(20, 20));
	    //indelScore.setValue(value);
	    
		final JLabel subLabel1 = new JLabel("subscore1:");
		add(subLabel1,"3,"+row);
		final JFormattedTextField subscore1 = new JFormattedTextField(formatter);
		//subscore1.setSize(new Dimension(10, 10));
		add(subscore1,"4,"+row);
		subscore1.setValue(1);
		
	  subscore1.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(subscore1.getText());
				subscore1.setValue(tmp);
				parameters.setSubScore1((Integer)subscore1.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
	  
		final JLabel evalLabel1 = new JLabel("evalScore1:");
		add(evalLabel1,"6,"+row);
		final JFormattedTextField evalScore1 = new JFormattedTextField(formatter);
		add(evalScore1,"7,"+row);
		evalScore1.setValue(10);
		
	  evalScore1.addFocusListener(new FocusListener() {
		
		public void focusLost(FocusEvent e) {
			int tmp = Integer.parseInt(evalScore1.getText());
			evalScore1.setValue(tmp);
			parameters.setEvalScore1((Integer)evalScore1.getValue());
		}
		
		public void focusGained(FocusEvent e) {
		}
	  });
	  
		row++;
		
		final JLabel subLabel2 = new JLabel("subscore2:");
		add(subLabel2,"3,"+row);
		final JFormattedTextField subscore2 = new JFormattedTextField(formatter);
		add(subscore2,"4,"+row);
		subscore2.setValue(-1);
		
	  subscore2.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(subscore2.getText());
				subscore2.setValue(tmp);
				parameters.setSubScore2((Integer)subscore2.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
	  
		final JLabel evalLabel2 = new JLabel("evalScore2:");
		add(evalLabel2,"6,"+row);
		final JFormattedTextField evalScore2 = new JFormattedTextField(formatter);
		add(evalScore2,"7,"+row);
		evalScore2.setValue(-10);
		row++;
		
      evalScore2.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(evalScore2.getText());
				evalScore2.setValue(tmp);
				parameters.setEvalScore1((Integer)evalScore2.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
		
		final JLabel subLabel3 = new JLabel("subscore3:");
		add(subLabel3,"3,"+row);
		final JFormattedTextField subscore3 = new JFormattedTextField(formatter);
		add(subscore3,"4,"+row);
		subscore3.setValue(-1);
	
		
	  subscore3.addFocusListener(new FocusListener() {
			
		public void focusLost(FocusEvent e) {
			int tmp = Integer.parseInt(subscore3.getText());
			subscore3.setValue(tmp);
			parameters.setSubScore3((Integer)subscore3.getValue());
		}
		
		public void focusGained(FocusEvent e) {
		}
	  });
		
		final JLabel evalLabel3 = new JLabel("evalScore3:");
		add(evalLabel3,"6,"+row);
		final JFormattedTextField evalScore3 = new JFormattedTextField(formatter);
		add(evalScore3,"7,"+row);
		evalScore3.setValue(-1);
		
	 evalScore3.addFocusListener(new FocusListener() {
			
		public void focusLost(FocusEvent e) {
			int tmp = Integer.parseInt(evalScore3.getText());
			evalScore3.setValue(tmp);
			parameters.setEvalScore1((Integer)evalScore3.getValue());
		}
		
		public void focusGained(FocusEvent e) {
		}
	 });
		
		row++;
		
		final JLabel evalLabel4 = new JLabel("evalScore4:");
		add(evalLabel4,"6,"+row);
		final JFormattedTextField evalScore4 = new JFormattedTextField(formatter);
		add(evalScore4,"7,"+row);
		evalScore4.setValue(-1);
		row+=2;
		
	 evalScore4.addFocusListener(new FocusListener() {
			
			public void focusLost(FocusEvent e) {
				int tmp = Integer.parseInt(evalScore4.getText());
				evalScore4.setValue(tmp);
				parameters.setEvalScore1((Integer)evalScore4.getValue());
			}
			
			public void focusGained(FocusEvent e) {
			}
		});
		
		//STEP 3 - DISCOVERY
//		final ProMTextArea header3 = new ProMTextArea(false);
		final JLabel header3 = new JLabel();
		header3.setText("Step 3: Process Discovery");
		add(header3,"0, "+row+","+ (size[0].length-1) +","+row);
//		header3.setFont(boldFont);
		row++;
		row++;		
		useAsHeader(header3);
		
		header3.setToolTipText("<html>Process Discovery with chosen algorithm. <br>"
				+ "If ASD is chosen, aligned log is used, else concise log is used. <br>"
				+ "ASD probably produces the best results.</html>");
		
		asdSlider = SlickerFactory.instance().createNiceDoubleSlider("Columnshare Discovery Threshold", 0,
				1, parameters.getDifferenceThreshold(), Orientation.HORIZONTAL);
		asdSlider.addChangeListener(new ChangeListener() {
			
			double val;
			
			public void stateChanged(ChangeEvent e) {
				val = asdSlider.getSlider().getValue();
				parameters.setAsdInclusionThreshold(val/10000);
			}

		});
		
		add(asdSlider,"0,"+(row+1)+"," + (size[0].length-1) +","+(row+1)); 
		asdSlider.getSlider().setValue(2500);
		row++;
		
		String[] algos = {"etm","alpha","inductive","alignedStepsDiscovery"};
		final JComboBox<String> selectAlgorithm = new JComboBox<String>(algos);
		selectAlgorithm.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String selection = (String)selectAlgorithm.getSelectedItem();
				parameters.setAlgorithm(selection);
				
				if(selection.equals("alignedStepsDiscovery"))
					asdSlider.setVisible(true);
				else
					asdSlider.setVisible(false);
					
				repaint();
			}

		});
//		selectAlgorithm.setPreferredSize(new Dimension(100, 30));
		final JLabel algoLabel = new JLabel("Discovery Algorithm:");
		add(algoLabel,"0,"+(row-1));
		add(selectAlgorithm,"1,"+(row-1)+", 4,"+(row-1));
		selectAlgorithm.setSelectedItem("alignedStepsDiscovery");
		row++;
		
		asdSlider.setToolTipText("sets the percentage needed by an event in a column to be included in the net.");
	

		
		//STEP 4 - NET ATTACHEMENT
//		final ProMTextArea header4 = new ProMTextArea(false);
		final JLabel header4 = new JLabel();
		header4.setText("Step 4: Net Enhancement");
		add(header4,"0, "+row+","+ (size[0].length-1) +","+row);
//		header3.setFont(boldFont);
		row++;
		
		useAsHeader(header4);
		
		header4.setToolTipText("<html>Alternative Predictions are attached to the event transitions. <br>"
				+ "Average confidence is added to event and alternative transitions.</html>");
		
		final JCheckBox enhancementCheckBox = SlickerFactory.instance().createCheckBox("Enhancement", true);
		parameters.setEnhancementActive(true);
		enhancementCheckBox.setSelected(parameters.isEnhancementActive());
		enhancementCheckBox.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				parameters.setEnhancementActive(enhancementCheckBox.isSelected());
				
				if(enhancementCheckBox.isSelected())
					probabilitySlider.setVisible(true);
				else
					probabilitySlider.setVisible(false);
				
				repaint();
			}
		});
		enhancementCheckBox.setOpaque(false);
//		checkBox.setPreferredSize(new Dimension(100, 30));
		add(enhancementCheckBox, "0, "+row+", 1, "+row);
		
		enhancementCheckBox.setToolTipText("Deactivate Preprocessing here, if log contains probability distributions over whole events instead of time windows.");
	
		
		
        probabilitySlider = SlickerFactory.instance().createNiceDoubleSlider("Alternative Probability Difference Threshold", 0,
				1, parameters.getDifferenceThreshold(), Orientation.HORIZONTAL);
		probabilitySlider.addChangeListener(new ChangeListener() {
			
			double val;
			
			public void stateChanged(ChangeEvent e) {
				val = probabilitySlider.getSlider().getValue();
				parameters.setDifferenceThreshold(val/10000);
			}

		});
		add(probabilitySlider,"2,"+row+ "," + (size[0].length-1) + ","+row); 
		probabilitySlider.getSlider().setValue((3000));
		
		probabilitySlider.setToolTipText("<html>sets for each event in the net which alternatives to show. <br>"
				+ "Only if the difference between the first prediction and an alternative is below this value, the alternative is shown in the net.</html>");
		
		
		
		//set TraceAlignment tooltips
		
	    indelLabel.setToolTipText("influencing the score for ADDING a gap-sign WHILE aligning");

		subLabel1.setToolTipText("influencing the score for NOT ADDING a gap-sign WHILE aligning, when events are equal (A A)");
		subLabel2.setToolTipText("influencing the score for NOT ADDING a gap-sign WHILE aligning, when events are different (A B)");
		subLabel3.setToolTipText("influencing the score for NOT ADDING a gap-sign WHILE aligning, when at least one event is a gap-sign (A -,-A or -,-)");
		
		evalLabel1.setToolTipText("influencing the score of an alignment AFTER the alignment, for each equal event (A A)");
		evalLabel2.setToolTipText("influencing the score of an alignment AFTER the alignment, for each different event (A B)");
		evalLabel3.setToolTipText("influencing the score of an alignment AFTER the alignment, for each non-gap sign sequence of 2 in one trace and a gap sign in the other");
		evalLabel4.setToolTipText("influencing the score of an alignment AFTER the alignment, if none of the others is true");
	}

	private void useAsHeader(JLabel l) {
		l.setOpaque(true);
		l.setBackground(Color.DARK_GRAY);
		l.setForeground(Color.WHITE);
		l.setHorizontalAlignment(SwingConstants.CENTER);
		Border border = l.getBorder();
		Border margin = new EmptyBorder(0,0,0,0);
		l.setBorder(new CompoundBorder(border, margin));
	}
	
}