package org.processmining.uncertaintymarker.models;

import java.util.ArrayList;

import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.uncertaintymarker.algorithms.TraceAlignment;

/**A data structure object that is used to store information for the process of {@link TraceAlignment}.
 * It contains the names of its predecessor alignments (or the original traces if it is on the first hierarchy level).
 * Furthermore it the result of the alignment in alignedTraces, which consists of the traces in its predecessors rearranged by adding 
 * "skips". In addition to this it stores the result of its evaluation, computed by using the scoring functions of the TraceAlignment with the Scoringparameters from the users input setting.
 * 
 * @author Oliver
 *
 */
public class Alignment {

	String name;
	String predecessor_1;
	String predecessor_2;
	ArrayList<XTrace> alignedTraces;
	int evaluation;
	
	public Alignment(String name,String id_part_1,String id_part_2,ArrayList<XTrace> alignedTraces){
		this.name = name;
		this.predecessor_1 = id_part_1;
		this.predecessor_2 = id_part_2;
		this.alignedTraces = alignedTraces;
	}

	public String getPredecessor_1() {
		return predecessor_1;
	}

	public void setPredecessor_1(String id_part_1) {
		this.predecessor_1 = id_part_1;
	}

	public String getPredecessor_2() {
		return predecessor_2;
	}

	public void setPredecessor_2(String id_part_2) {
		this.predecessor_2 = id_part_2;
	}

	public ArrayList<XTrace> getAlignedTraces() {
		return alignedTraces;
	}

	public void setAlignedTraces(ArrayList<XTrace> alignedTraces) {
		this.alignedTraces = alignedTraces;
	}

	public int getEvaluation() {
		return evaluation;
	}

	public void setEvaluation(int evaluation) {
		this.evaluation = evaluation;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	@Override
	public String toString(){
		String out = "";
		for(XTrace t: alignedTraces){
			for(XEvent e:t){
				out += " " + e.getAttributes().get("concept:name");
			}
			out+="\n";
		}
		return out;
	}
	
}
