package org.processmining.uncertaintymarker.models;

import java.util.ArrayList;

import org.processmining.uncertaintymarker.help.Utilities;


/**An Alternative stores information about the confidence of a certain prediction class.
 * Each Alternative belongs to a {@link Chunk}, in which it is stored within a collection among alternatives belonging to other prediction classes.
 * The Alternative contains a label, the average value of all occurrences within the whole log as well as a list containing all these occurrences.
 * 
 * @author Oliver
 *
 */
public class Alternative implements Comparable<Alternative>{

	String label;
	ArrayList<Double> list;
	double avg;
	
	public Alternative(String label, ArrayList<Double> list) {
		this.label = label;
		this.list = list;
	}
	
	/** Computes average probability of the alternative, rounds it to 3 digits and stores it in the avg attribute. 
	 *  Invoked after Chunk is full.  */
	public void computeAvg(int elementsInChunk){
		
		double cumProbs = 0;
		
		for(double p : list){
			cumProbs += p;
		}
		
		avg = Utilities.round(cumProbs/list.size(),3);
	}

	public int compareTo(Alternative o) {
		if(this.avg>o.avg){
			return 1;
		}
		else{
			return -1;
		}
		
	}

	public double getAvg() {
		return avg;
	}

	public void setAvg(double avg) {
		this.avg = avg;
	}
	
	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public ArrayList<Double> getList() {
		return list;
	}

	public void setList(ArrayList<Double> list) {
		this.list = list;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((label == null) ? 0 : label.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Alternative other = (Alternative) obj;
		if (label == null) {
			if (other.label != null)
				return false;
		} else if (!label.equals(other.label))
			return false;
		return true;
	}
	
	@Override
	public String toString(){
		String output = "\n" + this.getLabel() + " " + Utilities.round(this.getAvg(), 2) + " " +this.getList().toString();
		return output;
	}

	
}
