package org.processmining.uncertaintymarker.models;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Map;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.processmining.uncertaintymarker.algorithms.Consolidation;
import org.processmining.uncertaintymarker.help.Utilities;


public class SuccessorMatrix {

	public ArrayList<Map.Entry<String, ArrayList<Map.Entry<String,Double>> >> successorTable;
	
	public SuccessorMatrix(XLog log){
		
		String prev ="";
		String curr="";
		
		
		//create table
		successorTable = new ArrayList<>();
		XAttributeMapImpl mp  = (XAttributeMapImpl) log.get(0).get(0).getAttributes();
		for (Map.Entry<String, XAttribute> entry : mp.entrySet()){
			if(entry.getValue() instanceof XAttributeContinuousImpl){
				String columnLAbel = entry.getKey();
				ArrayList<Map.Entry<String,Double>> list = new ArrayList<>();
				Map.Entry<String, ArrayList< Map.Entry<String,Double> > > tuple = new AbstractMap.SimpleEntry(columnLAbel,list);
				successorTable.add(tuple);
			}
		}

		Consolidation c= new Consolidation();

		//for each trace
		for(XTrace trace: log){
			int rowCount=0;
			
			//for each row
			for(XEvent row: trace){
				
				//get current maximum
				XAttributeMapImpl mapOfRow = (XAttributeMapImpl) row.getAttributes();
				ArrayList<XAttributeContinuousImpl> maxima = c.findMaxima(mapOfRow);
				curr = maxima.get(0).getKey();
				
				if(rowCount==0){
					prev= curr;
					rowCount++;
					continue;
				}
				
				boolean entryPairExists=false;
				
				
				for(Map.Entry<String,ArrayList<Map.Entry<String,Double>>> entry: successorTable){
					if(entry.getKey().equals(prev)){
						for(Map.Entry<String,Double> entry2 : entry.getValue()){
							if(entry2.getKey().equals(curr)){
								entry2.setValue(entry2.getValue()+1);
								entryPairExists = true;
								break;
							}								
						}
						if(!entryPairExists){
							entry.getValue().add(new AbstractMap.SimpleEntry(curr, 1.0));
						}
						break;
					}
				}
				
				rowCount++;
				
				prev = curr;
			}
		}
		System.out.println(successorTable);
		
		//normalize
		for(Map.Entry<String,ArrayList<Map.Entry<String,Double>>> entry: successorTable){
			double sum = 0.0;
			for(Map.Entry<String,Double> entry2 : entry.getValue()){
				sum += entry2.getValue();

			}
			for(Map.Entry<String,Double> entry2 : entry.getValue()){
				entry2.setValue(Utilities.round(entry2.getValue()/sum,3));
			}
		}
		
		System.out.println(successorTable);
	}
}
