package org.processmining.uncertaintymarker.parameters;

import org.deckfour.xes.model.XLog;
import org.processmining.basicutils.parameters.impl.PluginParametersImpl;

public class Parameters extends PluginParametersImpl {

	//step 1 :Preprocessing
	private boolean chunkingActive;
	private int chunkMinimum;
	private String pruningAlgorithm;

	//step 2 : Trace Alignment
	private int indelScore;
	
	private int subScore1;
	private int subScore2;
	private int subScore3;

	private int evalScore1;
	private int evalScore2;
	private int evalScore3;
	private int evalScore4;
	
	//Step 3 : Process Discovery
	private String algorithm;
	private double asdInclusionThreshold;

	//Step 4 : Net Enhancement
	private double differenceThreshold;
	private boolean enhancementActive;
	
	public Parameters(XLog input1) {
		super();
//		setYourBoolean(input1.equals(input2));
//		setYourInteger(input1.toString().length() - input2.toString().length());
//		setYourString(input1.toString() + input2.toString());
	}

	public Parameters(Parameters parameters) {
		super(parameters);
		setChunkingActive(parameters.isChunkingActive());
		if(parameters.isChunkingActive()){
			setChunkMinimum(parameters.getChunkMinimum());
		}
		setDifferenceThreshold(parameters.getDifferenceThreshold());

		setIndelScore(parameters.getIndelScore());
		setSubScore1(parameters.getSubScore1());
		setSubScore2(parameters.getSubScore2());
		setSubScore3(parameters.getSubScore3());
		setEvalScore1(parameters.getEvalScore1());
		setEvalScore2(parameters.getEvalScore2());
		setEvalScore3(parameters.getEvalScore3());
		setEvalScore4(parameters.getEvalScore4());
		
		setAlgorithm(parameters.getAlgorithm());
		setAsdInclusionThreshold(parameters.getAsdInclusionThreshold());
		
		setEnhancementActive(parameters.isEnhancementActive());
	}

	public boolean equals(Object object) {
		if (object instanceof Parameters) {
			Parameters parameters = (Parameters) object;
			return super.equals(parameters) &&
					isChunkingActive() == parameters.isChunkingActive() &&
					getDifferenceThreshold()== parameters.getDifferenceThreshold() &&
					getIndelScore() == parameters.getIndelScore() &&
					getSubScore1() == parameters.getSubScore1() &&
					getSubScore2() == parameters.getSubScore2() &&
					getSubScore3() == parameters.getSubScore3() &&
					getEvalScore1() == parameters.getEvalScore1() &&
					getEvalScore2() == parameters.getEvalScore2() &&
					getEvalScore3() == parameters.getEvalScore3() &&
					getEvalScore4() == parameters.getEvalScore4() &&
					getAlgorithm().equals(parameters.getAlgorithm());
		}
		return false;
	}
	
	public String getAlgorithm() {
		return algorithm;
	}
	
	public String toString() {
		return "(" + getAlgorithm() + "," + "," + isChunkingActive() + ")";
	}
	
	public double getDifferenceThreshold() {
		return differenceThreshold;
	}

	public void setDifferenceThreshold(double differenceThreshold) {
		this.differenceThreshold = differenceThreshold;
	}

	public void setAlgorithm(String selectedItem) {
		algorithm = selectedItem;
	}

	public double getAsdInclusionThreshold() {
		return asdInclusionThreshold;
	}

	public void setAsdInclusionThreshold(double asdInclusionThreshold) {
		this.asdInclusionThreshold = asdInclusionThreshold;
	}

	public int getEvalScore4() {
		return evalScore4;
	}

	public void setEvalScore4(int evalScore4) {
		this.evalScore4 = evalScore4;
	}

	public int getEvalScore3() {
		return evalScore3;
	}

	public void setEvalScore3(int evalScore3) {
		this.evalScore3 = evalScore3;
	}

	public int getEvalScore2() {
		return evalScore2;
	}

	public void setEvalScore2(int evalScore2) {
		this.evalScore2 = evalScore2;
	}

	public int getEvalScore1() {
		return evalScore1;
	}

	public void setEvalScore1(int evalScore1) {
		this.evalScore1 = evalScore1;
	}

	public int getSubScore3() {
		return subScore3;
	}

	public void setSubScore3(int subScore3) {
		this.subScore3 = subScore3;
	}

	public int getSubScore2() {
		return subScore2;
	}

	public void setSubScore2(int subScore2) {
		this.subScore2 = subScore2;
	}

	public int getSubScore1() {
		return subScore1;
	}

	public void setSubScore1(int subScore1) {
		this.subScore1 = subScore1;
	}

	public int getIndelScore() {
		return indelScore;
	}

	public void setIndelScore(int indelScore) {
		this.indelScore = indelScore;
	}

	public boolean isChunkingActive() {
		return chunkingActive;
	}

	public void setChunkingActive(boolean chunking) {
		this.chunkingActive = chunking;
	}

	public String getPruningAlgorithm() {
		return pruningAlgorithm;
	}

	public void setPruningAlgorithm(String pruningAlgorithm) {
		this.pruningAlgorithm = pruningAlgorithm;
	}

	public void setChunkMinimum(Integer value) {
		chunkMinimum=value;
	}

	public int getChunkMinimum() {
		return chunkMinimum;
	}

	public boolean isEnhancementActive() {
		return enhancementActive;
	}

	public void setEnhancementActive(boolean enhancementActive) {
		this.enhancementActive = enhancementActive;
	}

	public void setChunkMinimum(int chunkMinimum) {
		this.chunkMinimum = chunkMinimum;
	}
}
