package org.processmining.uncertaintymarker.plugins;

import java.util.Collection;

import javax.swing.JPanel;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.uncertaintymarker.algorithms.MainAlgorithm;
import org.processmining.uncertaintymarker.connections.Connection;
import org.processmining.uncertaintymarker.dialogs.Explanation;
import org.processmining.uncertaintymarker.dialogs.UserInput;
import org.processmining.uncertaintymarker.help.DescriptionText;
import org.processmining.uncertaintymarker.parameters.Parameters;

@Plugin(name = "Uncertainty Marker", parameterLabels = { "Probability Log"}, 
	    returnLabels = { "Uncertainty Net","Petri Net", "Concise Log","Aligned Log"}, returnTypes = { Petrinet.class,Petrinet.class, XLog.class, XLog.class}, help = DescriptionText.TEXT)
public class UncertaintyMarker extends MainAlgorithm {
	//before: your plugin
	/**
	 * The plug-in variant that runs in any context and requires a parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @param parameters The parameters to use.
	 * @return The output.
	 */
//	@UITopiaVariant(affiliation = "University of Mannheim", author = "Oliver Meier", email = "omeier@mail.uni-mannheim.de")
//	@PluginVariant(variantLabel = "testi, parameters", requiredParameterLabels = { 0})
//	public Object[] run(PluginContext context, XLog input1, YourParameters parameters) {
//		// Apply the algorithm depending on whether a connection already exists.
//	    return runConnections(context, input1, parameters);
//	}
	
	/**
	 * The plug-in variant that runs in any context and uses the default parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
//	@UITopiaVariant(affiliation = "University of Mannheim", author = "Oliver Meier", email = "omeier@mail.uni-mannheim.de")
//	@PluginVariant(variantLabel = "test2, parameters", requiredParameterLabels = { 0})
//	public Object[] runDefault(PluginContext context, XLog input1) {
//		// Get the default parameters.
//	    YourParameters parameters = new YourParameters(input1);
//		// Apply the algorithm depending on whether a connection already exists.
//	    return runConnections(context, input1, parameters);
//	}
	
	/**
	 * The plug-in variant that runs in a UI context and uses a dialog to get the parameters.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
	@UITopiaVariant(affiliation = "University of Mannheim", author = "Oliver Meier", email = "omeier@mail.uni-mannheim.de")
	@PluginVariant(variantLabel = "test, dialog", requiredParameterLabels = {0})
	public Object[] runUI(UIPluginContext context, XLog input1) {
		// Get the default parameters.
	    Parameters parameters = new Parameters(input1);
	    // Get a dialog for this parameters.
	    Explanation exp = new Explanation();
	    InteractionResult result = context.showWizard("Uncertainty Marker: Explanation", true, false, exp);
	    UserInput dialog = new UserInput(context, input1, parameters);
	    
	    while(!(result == InteractionResult.FINISHED || result == InteractionResult.CANCEL)){
	    	if(result == InteractionResult.NEXT)
	    		result = context.showWizard("Uncertainty Marker: Settings", false, true, dialog);
	    	if(result == InteractionResult.PREV)
	    		result = context.showWizard("Uncertainty Marker: Explanation", true, false, exp);
	    }
	    
	    if (result == InteractionResult.FINISHED) {
	    	return runConnections(context, input1, parameters);
	    }
	    	return null;
	}	
	
	/**
	 * The plug-in variant that allows one to test the dialog to get the parameters.
	 * 
	 * @param context The context to run in.
	 * @return The output.
	 */
//	@UITopiaVariant(affiliation = "University of Mannheim", author = "Oliver Meier", email = "omeier@mail.uni-mannheim.de")
//	@PluginVariant(variantLabel = "Your plug-in name, dialog", requiredParameterLabels = { })
//	public Object[] testUI(UIPluginContext context) {
//		// Create default inputs.
//		XLog input1 = null;
//		// Get the default parameters.
//	    YourParameters parameters = null; //new YourParameters(input1);
//	    // Get a dialog for this parameters.
//	    YourDialog dialog = new YourDialog(context, input1, parameters);
//	    // Show the dialog. User can now change the parameters.
//	    InteractionResult result = context.showWizard("Your dialog titlelelele", true, true, dialog);
//	    // User has close the dialog.
//	    if (result == InteractionResult.FINISHED) {
//			// Apply the algorithm depending on whether a connection already exists.
//	    	return runConnections(context, input1, parameters);
//	    }
//	    // Dialog got canceled.
//	    return null;
//	}	
	
	/**
	 * Apply the algorithm depending on whether a connection already exists.
	 * 
	 * @param context The context to run in.
	 * @param input1 The first input.
	 * @param input2 The second input.
	 * @return The output.
	 */
	private Object[] runConnections(PluginContext context, XLog input1, Parameters parameters) {
		if (parameters.isTryConnections()) {
			// Try to found a connection that matches the inputs and the parameters.
			Collection<Connection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						Connection.class, context, input1);
				for (Connection connection : connections) {
//					if (connection.getObjectWithRole(YourConnection.FIRSTINPUT).equals(input1) &&  connection.getParameters().equals(parameters)) {
					if (connection.getObjectWithRole(connection.FIRSTINPUT).equals(input1) &&  connection.getParameters().equals(parameters)) {
						// Found a match. Return the associated output as result of the algorithm.
						return connection
								.getObjectWithRole(connection.OUTPUT3);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		// No connection found. Apply the algorithm to compute a fresh output result.
		Object[] output = apply(context, input1, parameters);
		if (parameters.isTryConnections()) {
			// Store a connection containing the inputs, output, and parameters.
			context.getConnectionManager().addConnection(
					new Connection(input1, (Petrinet) output[0],(Petrinet) output[1],(XLog) output[2],(XLog) output[3],parameters));
		}
		
//		// Return the output.
		return output;
	}
}